// Copyright 2021 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package fuzzing_event_engine;

message Actions {
    // Map of tick event (0..??) to the amount to increment the timer by.
    repeated Delay tick_lengths = 1;
    // Map of task id to the amount to delay execution of the task by.
    repeated Delay run_delay = 2;
};

message Delay {
    uint32 id = 1;
    uint64 delay_us = 2;
};
