//
// Copyright 2022 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#ifndef GRPC_INTERNAL_CPP_ORCA_ORCA_INTERCEPTOR_H
#define GRPC_INTERNAL_CPP_ORCA_ORCA_INTERCEPTOR_H

#include <grpcpp/support/interceptor.h>
#include <grpcpp/support/server_interceptor.h>

namespace grpc {

class ServerBuilder;

namespace experimental {
class ServerRpcInfo;

class OrcaServerInterceptor : public Interceptor {
 public:
  explicit OrcaServerInterceptor(ServerRpcInfo* info) : info_(info) {}

  void Intercept(InterceptorBatchMethods* methods) override;

 private:
  ServerRpcInfo* info_;
};

class OrcaServerInterceptorFactory : public ServerInterceptorFactoryInterface {
 public:
  static void Register(ServerBuilder* builder);
  Interceptor* CreateServerInterceptor(ServerRpcInfo* info) override;
};

}  // namespace experimental
}  // namespace grpc

#endif  // GRPC_INTERNAL_CPP_ORCA_ORCA_INTERCEPTOR_H
