# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/metrics.proto

require 'google/protobuf'

Google::Protobuf::DescriptorPool.generated_pool.build do
  add_message "grpc.testing.GaugeResponse" do
    optional :name, :string, 1
    oneof :value do
      optional :long_value, :int64, 2
      optional :double_value, :double, 3
      optional :string_value, :string, 4
    end
  end
  add_message "grpc.testing.GaugeRequest" do
    optional :name, :string, 1
  end
  add_message "grpc.testing.EmptyMessage" do
  end
end

module Grpc
  module Testing
    GaugeResponse = Google::Protobuf::DescriptorPool.generated_pool.lookup("grpc.testing.GaugeResponse").msgclass
    GaugeRequest = Google::Protobuf::DescriptorPool.generated_pool.lookup("grpc.testing.GaugeRequest").msgclass
    EmptyMessage = Google::Protobuf::DescriptorPool.generated_pool.lookup("grpc.testing.EmptyMessage").msgclass
  end
end
