# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: client_control.proto

require 'google/protobuf'

Google::Protobuf::DescriptorPool.generated_pool.build do
  add_file("client_control.proto", :syntax => :proto3) do
    add_message "client_control.DoEchoRpcRequest" do
      optional :request, :string, 1
    end
    add_message "client_control.Void" do
    end
    add_message "client_control.Port" do
      optional :port, :int32, 1
    end
  end
end

module ClientControl
  DoEchoRpcRequest = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("client_control.DoEchoRpcRequest").msgclass
  Void = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("client_control.Void").msgclass
  Port = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("client_control.Port").msgclass
end
