// Copyright 2022 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef TEST_H
#define TEST_H

#include <stdlib.h>
#include <sys/resource.h>
#include <sys/time.h>

struct MemStats {
  long rss;  // Resident set size, in kb
  static MemStats Snapshot() {
    struct rusage usage;
    if (0 != getrusage(RUSAGE_SELF, &usage)) abort();
    return MemStats{usage.ru_maxrss};
  }
};

#endif
