#! /bin/bash
# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Checks if any file contains "DO NOT SUBMIT"

cd "$(dirname "$0")/../../.." || exit 1
git grep -Irn 'DO NOT SUBMIT' -- \
  './*' \
  ':!*check_do_not_submit.sh' \
  ':!third_party/'
test $? -eq 1 || exit 1
