/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     src/proto/grpc/health/v1/health.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "src/proto/grpc/health/v1/health.upb.h"

#include "upb/port_def.inc"

static const upb_MiniTable_Field grpc_health_v1_HealthCheckRequest__fields[1] = {
  {1, UPB_SIZE(0, 0), UPB_SIZE(0, 0), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc_health_v1_HealthCheckRequest_msginit = {
  NULL,
  &grpc_health_v1_HealthCheckRequest__fields[0],
  UPB_SIZE(8, 24), 1, kUpb_ExtMode_NonExtendable, 1, 255, 0,
};

static const upb_MiniTable_Field grpc_health_v1_HealthCheckResponse__fields[1] = {
  {1, UPB_SIZE(0, 0), UPB_SIZE(0, 0), kUpb_NoSub, 5, kUpb_FieldMode_Scalar | (kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable grpc_health_v1_HealthCheckResponse_msginit = {
  NULL,
  &grpc_health_v1_HealthCheckResponse__fields[0],
  UPB_SIZE(4, 8), 1, kUpb_ExtMode_NonExtendable, 1, 255, 0,
};

static const upb_MiniTable *messages_layout[2] = {
  &grpc_health_v1_HealthCheckRequest_msginit,
  &grpc_health_v1_HealthCheckResponse_msginit,
};

const upb_MiniTable_File src_proto_grpc_health_v1_health_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  2,
  0,
  0,
};

#include "upb/port_undef.inc"

