// Copyright 2021 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
syntax = "proto3";

package chunked_vector_fuzzer;

message EmplaceBack {
    int32 vector = 1;
    int32 value = 2;
}

message PopBack {
    int32 vector = 1;
}

message Copy {
    int32 from = 1;
    int32 to = 2;
}

message ClearVector {
    int32 vector = 1;
}

message RemoveIf {
    int32 vector = 1;
    int32 value = 2;
}

message Action {
    oneof action_type {
        EmplaceBack emplace_back = 1;
        PopBack pop_back = 2;
        Copy copy = 3;
        Copy move = 4;
        ClearVector clear = 5;
        Copy swap = 6;
        RemoveIf remove_if = 7;
    }
}

message Msg {
    repeated Action actions = 1;
}
