// Copyright 2021 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// We would ideally just use the proto file from the googleapis submodule, but
// that breaks the expectations of the CMake build system - gRPC C++ needs to
// stay buildable/installable even if the submodules are not present (e.g. the
// tar.gz archive with gRPC doesn't contain the submodules), assuming that the
// dependencies are already installed. Given that this is needed for a test we
// could probably say that this test does not build for CMake but that way we
// would lose the ability to test xds_end2end_test for portability.

syntax = "proto3";

package google.api.expr.v1alpha1;

message Expr {}

message CheckedExpr {}
