// Copyright 2021 The gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

option java_multiple_files = true;
option java_package = "io.grpc.examples.subpackage";
option java_outer_classname = "SubpackageProto";
option objc_class_prefix = "SP";

package subpackage;

import "proto/my.proto";

message Subpackaged {
  string name = 1;
  somewhere_else.MyMessage msg = 2;
}
