# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/benchmark_service.proto

require 'google/protobuf'

require 'src/proto/grpc/testing/messages_pb'

Google::Protobuf::DescriptorPool.generated_pool.build do
  add_file("src/proto/grpc/testing/benchmark_service.proto", :syntax => :proto3) do
  end
end

module Grpc
  module Testing
  end
end
