#!/usr/bin/env bash
# Copyright 2021 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

set -ex

cd "$(dirname "$0")"

GITHUB_URL="https://github.com"
REPO="bazelbuild/bazel"

VERSION=$(curl -Ls "${GITHUB_URL}/${REPO}/releases/latest" | \
          grep "href=.*\.tar.gz" | \
          grep -o "[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*")

./test_single_bazel_version.sh "$VERSION"
