// Copyright 2021 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package hpack_parser_fuzzer;

message Empty {}

message Frame {
  bool end_of_headers = 1;
  bool end_of_stream = 2;
  bool priority = 3;
  int32 stop_buffering_after_segments = 4;
  int32 max_metadata_length = 5;
  repeated bytes parse = 6;
}

message Msg {
  repeated Frame frames = 2;
}

