<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/control.proto

namespace Grpc\Testing;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\RepeatedField;
use Google\Protobuf\Internal\GPBUtil;

/**
 * Generated from protobuf message <code>grpc.testing.LoadParams</code>
 */
class LoadParams extends \Google\Protobuf\Internal\Message
{
    protected $load;

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type \Grpc\Testing\ClosedLoopParams $closed_loop
     *     @type \Grpc\Testing\PoissonParams $poisson
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\Src\Proto\Grpc\Testing\Control::initOnce();
        parent::__construct($data);
    }

    /**
     * Generated from protobuf field <code>.grpc.testing.ClosedLoopParams closed_loop = 1;</code>
     * @return \Grpc\Testing\ClosedLoopParams
     */
    public function getClosedLoop()
    {
        return $this->readOneof(1);
    }

    /**
     * Generated from protobuf field <code>.grpc.testing.ClosedLoopParams closed_loop = 1;</code>
     * @param \Grpc\Testing\ClosedLoopParams $var
     * @return $this
     */
    public function setClosedLoop($var)
    {
        GPBUtil::checkMessage($var, \Grpc\Testing\ClosedLoopParams::class);
        $this->writeOneof(1, $var);

        return $this;
    }

    /**
     * Generated from protobuf field <code>.grpc.testing.PoissonParams poisson = 2;</code>
     * @return \Grpc\Testing\PoissonParams
     */
    public function getPoisson()
    {
        return $this->readOneof(2);
    }

    /**
     * Generated from protobuf field <code>.grpc.testing.PoissonParams poisson = 2;</code>
     * @param \Grpc\Testing\PoissonParams $var
     * @return $this
     */
    public function setPoisson($var)
    {
        GPBUtil::checkMessage($var, \Grpc\Testing\PoissonParams::class);
        $this->writeOneof(2, $var);

        return $this;
    }

    /**
     * @return string
     */
    public function getLoad()
    {
        return $this->whichOneof("load");
    }

}

