/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/metrics/v3/stats.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/config/metrics/v3/stats.upb.h"
#include "envoy/config/core/v3/address.upb.h"
#include "envoy/type/matcher/v3/string.upb.h"
#include "google/protobuf/any.upb.h"
#include "google/protobuf/wrappers.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_MiniTable_Sub envoy_config_metrics_v3_StatsSink_submsgs[1] = {
  {.submsg = &google_protobuf_Any_msginit},
};

static const upb_MiniTable_Field envoy_config_metrics_v3_StatsSink__fields[2] = {
  {1, UPB_SIZE(4, 8), UPB_SIZE(0, 0), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 24), UPB_SIZE(-1, -1), 0, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_StatsSink_msginit = {
  &envoy_config_metrics_v3_StatsSink_submsgs[0],
  &envoy_config_metrics_v3_StatsSink__fields[0],
  UPB_SIZE(16, 40), 2, kUpb_ExtMode_NonExtendable, 1, 255, 0,
};

static const upb_MiniTable_Sub envoy_config_metrics_v3_StatsConfig_submsgs[4] = {
  {.submsg = &envoy_config_metrics_v3_TagSpecifier_msginit},
  {.submsg = &google_protobuf_BoolValue_msginit},
  {.submsg = &envoy_config_metrics_v3_StatsMatcher_msginit},
  {.submsg = &envoy_config_metrics_v3_HistogramBucketSettings_msginit},
};

static const upb_MiniTable_Field envoy_config_metrics_v3_StatsConfig__fields[4] = {
  {1, UPB_SIZE(4, 8), UPB_SIZE(0, 0), 0, 11, kUpb_FieldMode_Array | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(8, 16), UPB_SIZE(1, 1), 1, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 24), UPB_SIZE(2, 2), 2, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(16, 32), UPB_SIZE(0, 0), 3, 11, kUpb_FieldMode_Array | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_StatsConfig_msginit = {
  &envoy_config_metrics_v3_StatsConfig_submsgs[0],
  &envoy_config_metrics_v3_StatsConfig__fields[0],
  UPB_SIZE(20, 40), 4, kUpb_ExtMode_NonExtendable, 4, 255, 0,
};

static const upb_MiniTable_Sub envoy_config_metrics_v3_StatsMatcher_submsgs[2] = {
  {.submsg = &envoy_type_matcher_v3_ListStringMatcher_msginit},
  {.submsg = &envoy_type_matcher_v3_ListStringMatcher_msginit},
};

static const upb_MiniTable_Field envoy_config_metrics_v3_StatsMatcher__fields[3] = {
  {1, UPB_SIZE(4, 8), UPB_SIZE(-1, -1), kUpb_NoSub, 8, kUpb_FieldMode_Scalar | (kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(4, 8), UPB_SIZE(-1, -1), 0, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(4, 8), UPB_SIZE(-1, -1), 1, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_StatsMatcher_msginit = {
  &envoy_config_metrics_v3_StatsMatcher_submsgs[0],
  &envoy_config_metrics_v3_StatsMatcher__fields[0],
  UPB_SIZE(8, 24), 3, kUpb_ExtMode_NonExtendable, 3, 255, 0,
};

static const upb_MiniTable_Field envoy_config_metrics_v3_TagSpecifier__fields[3] = {
  {1, UPB_SIZE(12, 24), UPB_SIZE(0, 0), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(4, 8), UPB_SIZE(-1, -1), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(4, 8), UPB_SIZE(-1, -1), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_TagSpecifier_msginit = {
  NULL,
  &envoy_config_metrics_v3_TagSpecifier__fields[0],
  UPB_SIZE(20, 40), 3, kUpb_ExtMode_NonExtendable, 3, 255, 0,
};

static const upb_MiniTable_Sub envoy_config_metrics_v3_HistogramBucketSettings_submsgs[1] = {
  {.submsg = &envoy_type_matcher_v3_StringMatcher_msginit},
};

static const upb_MiniTable_Field envoy_config_metrics_v3_HistogramBucketSettings__fields[2] = {
  {1, UPB_SIZE(4, 8), UPB_SIZE(1, 1), 0, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(8, 16), UPB_SIZE(0, 0), kUpb_NoSub, 1, kUpb_FieldMode_Array | kUpb_LabelFlags_IsPacked | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_HistogramBucketSettings_msginit = {
  &envoy_config_metrics_v3_HistogramBucketSettings_submsgs[0],
  &envoy_config_metrics_v3_HistogramBucketSettings__fields[0],
  UPB_SIZE(12, 24), 2, kUpb_ExtMode_NonExtendable, 2, 255, 0,
};

static const upb_MiniTable_Sub envoy_config_metrics_v3_StatsdSink_submsgs[1] = {
  {.submsg = &envoy_config_core_v3_Address_msginit},
};

static const upb_MiniTable_Field envoy_config_metrics_v3_StatsdSink__fields[3] = {
  {1, UPB_SIZE(4, 8), UPB_SIZE(-1, -1), 0, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(4, 8), UPB_SIZE(-1, -1), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 24), UPB_SIZE(0, 0), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_StatsdSink_msginit = {
  &envoy_config_metrics_v3_StatsdSink_submsgs[0],
  &envoy_config_metrics_v3_StatsdSink__fields[0],
  UPB_SIZE(20, 40), 3, kUpb_ExtMode_NonExtendable, 3, 255, 0,
};

static const upb_MiniTable_Sub envoy_config_metrics_v3_DogStatsdSink_submsgs[2] = {
  {.submsg = &envoy_config_core_v3_Address_msginit},
  {.submsg = &google_protobuf_UInt64Value_msginit},
};

static const upb_MiniTable_Field envoy_config_metrics_v3_DogStatsdSink__fields[3] = {
  {1, UPB_SIZE(20, 32), UPB_SIZE(-5, -5), 0, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(8, 8), UPB_SIZE(0, 0), kUpb_NoSub, 9, kUpb_FieldMode_Scalar | (kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(16, 24), UPB_SIZE(1, 1), 1, 11, kUpb_FieldMode_Scalar | (kUpb_FieldRep_Pointer << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_DogStatsdSink_msginit = {
  &envoy_config_metrics_v3_DogStatsdSink_submsgs[0],
  &envoy_config_metrics_v3_DogStatsdSink__fields[0],
  UPB_SIZE(24, 40), 3, kUpb_ExtMode_NonExtendable, 1, 255, 0,
};

static const upb_MiniTable_Field envoy_config_metrics_v3_HystrixSink__fields[1] = {
  {1, UPB_SIZE(0, 0), UPB_SIZE(0, 0), kUpb_NoSub, 3, kUpb_FieldMode_Scalar | (kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
};

const upb_MiniTable envoy_config_metrics_v3_HystrixSink_msginit = {
  NULL,
  &envoy_config_metrics_v3_HystrixSink__fields[0],
  UPB_SIZE(8, 8), 1, kUpb_ExtMode_NonExtendable, 1, 255, 0,
};

static const upb_MiniTable *messages_layout[8] = {
  &envoy_config_metrics_v3_StatsSink_msginit,
  &envoy_config_metrics_v3_StatsConfig_msginit,
  &envoy_config_metrics_v3_StatsMatcher_msginit,
  &envoy_config_metrics_v3_TagSpecifier_msginit,
  &envoy_config_metrics_v3_HistogramBucketSettings_msginit,
  &envoy_config_metrics_v3_StatsdSink_msginit,
  &envoy_config_metrics_v3_DogStatsdSink_msginit,
  &envoy_config_metrics_v3_HystrixSink_msginit,
};

const upb_MiniTable_File envoy_config_metrics_v3_stats_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  8,
  0,
  0,
};

#include "upb/port_undef.inc"

