/*
 *
 * Copyright 2015 gRPC authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#import "GRXNSScalarEnumerator.h"

@implementation GRXNSScalarEnumerator {
  id _value;
}

- (instancetype)init {
  return [self initWithValue:nil];
}

// Designated initializer.
- (instancetype)initWithValue:(id)value {
  if ((self = [super init])) {
    _value = value;
  }
  return self;
}

- (id)nextObject {
  id value = _value;
  _value = nil;
  return value;
}
@end
