# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/proxy-service.proto

require 'google/protobuf'

require 'src/proto/grpc/testing/control_pb'
require 'src/proto/grpc/testing/stats_pb'
Google::Protobuf::DescriptorPool.generated_pool.build do
  add_message "grpc.testing.ProxyStat" do
    optional :latency, :double, 1
  end
end

module Grpc
  module Testing
    ProxyStat = Google::Protobuf::DescriptorPool.generated_pool.lookup("grpc.testing.ProxyStat").msgclass
  end
end
