<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/control.proto

namespace Grpc\Testing;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\RepeatedField;
use Google\Protobuf\Internal\GPBUtil;

/**
 * presence of SecurityParams implies use of TLS
 *
 * Generated from protobuf message <code>grpc.testing.SecurityParams</code>
 */
class SecurityParams extends \Google\Protobuf\Internal\Message
{
    /**
     * Generated from protobuf field <code>bool use_test_ca = 1;</code>
     */
    protected $use_test_ca = false;
    /**
     * Generated from protobuf field <code>string server_host_override = 2;</code>
     */
    protected $server_host_override = '';
    /**
     * Generated from protobuf field <code>string cred_type = 3;</code>
     */
    protected $cred_type = '';

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type bool $use_test_ca
     *     @type string $server_host_override
     *     @type string $cred_type
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\Src\Proto\Grpc\Testing\Control::initOnce();
        parent::__construct($data);
    }

    /**
     * Generated from protobuf field <code>bool use_test_ca = 1;</code>
     * @return bool
     */
    public function getUseTestCa()
    {
        return $this->use_test_ca;
    }

    /**
     * Generated from protobuf field <code>bool use_test_ca = 1;</code>
     * @param bool $var
     * @return $this
     */
    public function setUseTestCa($var)
    {
        GPBUtil::checkBool($var);
        $this->use_test_ca = $var;

        return $this;
    }

    /**
     * Generated from protobuf field <code>string server_host_override = 2;</code>
     * @return string
     */
    public function getServerHostOverride()
    {
        return $this->server_host_override;
    }

    /**
     * Generated from protobuf field <code>string server_host_override = 2;</code>
     * @param string $var
     * @return $this
     */
    public function setServerHostOverride($var)
    {
        GPBUtil::checkString($var, True);
        $this->server_host_override = $var;

        return $this;
    }

    /**
     * Generated from protobuf field <code>string cred_type = 3;</code>
     * @return string
     */
    public function getCredType()
    {
        return $this->cred_type;
    }

    /**
     * Generated from protobuf field <code>string cred_type = 3;</code>
     * @param string $var
     * @return $this
     */
    public function setCredType($var)
    {
        GPBUtil::checkString($var, True);
        $this->cred_type = $var;

        return $this;
    }

}

